@@ -276,7 +276,7 @@ static retvalue configparser_ ## sname ## _set_ ## field(UNUSED(void *dummy), UN
 	r = config_getall(iter, &formula); \
 	if( ! RET_IS_OK(r) ) \
 		return r; \
-	r = term_compile(&item->field, formula, T_OR|T_BRACKETS|T_NEGATION|T_VERSION|T_NOTEQUAL); \
+	r = term_compilefortargetdecision(&item->field, formula); \
 	free(formula); \
 	return r; \
 }
@@ -288,7 +288,7 @@ static retvalue configparser_ ## sname ## _set_ ## field(UNUSED(void *dummy), UN
 	r = config_getall(iter, &formula); \
 	if( ! RET_IS_OK(r) ) \
 		return r; \
-	r = term_compile(&item->field, formula, T_OR|T_BRACKETS|T_NEGATION|T_VERSION|T_NOTEQUAL); \
+	r = term_compilefortargetdecision(&item->field, formula); \
 	free(formula); \
 	item->field ## _set = true; \
 	return r; \
