@@ -54,8 +54,6 @@
 static void database_free(/*@only@*/ struct database *db) {
 	if( db == NULL )
 		return;
-	free(db->directory);
-	free(db->filesdir);
 	free(db->version);
 	free(db->lastsupportedversion);
 	free(db->dbversion);
@@ -63,6 +61,10 @@ static void database_free(/*@only@*/ struct database *db) {
 	free(db);
 }
 
+static inline char *dbfilename(const char *filename) {
+	return calc_dirconcat(global.dbdir, filename);
+}
+
 /**********************/
 /* lock file handling */
 /**********************/
@@ -75,11 +77,11 @@ static retvalue database_lock(struct database *db, size_t waitforlock) {
 
 	assert( !db->locked );
 	db->dircreationdepth = 0;
-	r = dir_create_needed(db->directory, &db->dircreationdepth);
+	r = dir_create_needed(global.dbdir, &db->dircreationdepth);
 	if( RET_WAS_ERROR(r) )
 		return r;
 
-	lockfile = calc_dirconcat(db->directory, "lockfile");
+	lockfile = dbfilename("lockfile");
 	if( lockfile == NULL )
 		return RET_ERROR_OOM;
 	fd = open(lockfile,O_WRONLY|O_CREAT|O_EXCL|O_NOFOLLOW|O_NOCTTY,S_IRUSR|S_IWUSR);
@@ -129,7 +131,7 @@ static void releaselock(struct database *db) {
 
 	assert( db->locked );
 
-	lockfile = calc_dirconcat(db->directory, "lockfile");
+	lockfile = dbfilename("lockfile");
 	if( lockfile == NULL )
 		return;
 	if( unlink(lockfile) != 0 ) {
@@ -139,7 +141,7 @@ static void releaselock(struct database *db) {
 		(void)unlink(lockfile);
 	}
 	free(lockfile);
-	dir_remove_new(db->directory, db->dircreationdepth);
+	dir_remove_new(global.dbdir, db->dircreationdepth);
 	db->locked = false;
 }
 
@@ -176,10 +178,10 @@ retvalue database_close(struct database *db) {
 	return result;
 }
 
-static retvalue database_hasdatabasefile(const struct database *database, const char *filename, /*@out@*/bool *exists_p) {
+static retvalue database_hasdatabasefile(UNUSED(const struct database *database), const char *filename, /*@out@*/bool *exists_p) {
 	char *fullfilename;
 
-	fullfilename = calc_dirconcat(database->directory, filename);
+	fullfilename = dbfilename(filename);
 	if( fullfilename == NULL )
 		return RET_ERROR_OOM;
 	*exists_p = isregularfile(fullfilename);
@@ -187,12 +189,6 @@ static retvalue database_hasdatabasefile(const struct database *database, const
 	return RET_OK;
 }
 
-const char *database_directory(struct database *database) {
-	/* this has to make sure the database directory actually
-	 * exists. Though locking before already does so */
-	return database->directory;
-}
-
 enum database_type {
 	dbt_QUERY,
 	dbt_BTREE, dbt_BTREEDUP, dbt_BTREEPAIRS,
@@ -207,12 +203,12 @@ static const uint32_t types[dbt_COUNT] = {
 
 static int paireddatacompare(UNUSED(DB *db), const DBT *a, const DBT *b);
 
-static retvalue database_opentable(struct database *database, const char *filename, /*@null@*/const char *subtable, enum database_type type, uint32_t flags, /*@out@*/DB **result) {
+static retvalue database_opentable(UNUSED(struct database *database), const char *filename, /*@null@*/const char *subtable, enum database_type type, uint32_t flags, /*@out@*/DB **result) {
 	char *fullfilename;
 	DB *table;
 	int dbret;
 
-	fullfilename = calc_dirconcat(database_directory(database), filename);
+	fullfilename = dbfilename(filename);
 	if( fullfilename == NULL )
 		return RET_ERROR_OOM;
 
@@ -345,12 +341,12 @@ retvalue database_listsubtables(struct database *database,const char *filename,s
 	}
 }
 
-retvalue database_dropsubtable(struct database *database, const char *table, const char *subtable) {
+retvalue database_dropsubtable(UNUSED(struct database *database), const char *table, const char *subtable) {
 	char *filename;
 	DB *db;
 	int dbret;
 
-	filename = calc_dirconcat(database->directory, table);
+	filename = dbfilename(table);
 	if( filename == NULL )
 		return RET_ERROR_OOM;
 
@@ -422,10 +418,49 @@ static retvalue warnidentifers(struct database *db, const struct strlist *identi
 		(void)fputs("To ignore use --ignore=undefinedtarget.\n", stderr);
 		return RET_ERROR;
 	}
+	if( readonly )
+		return RET_OK;
 	for( d = distributions ; d != NULL ; d = d->next ) {
-		// TODO: check for new architectures here and warn then...
-		if( readonly )
-			continue;
+		bool architecture_existed[d->architectures.count];
+		bool have_old = false;
+
+		/* check for new architectures */
+		memset(architecture_existed, 0, sizeof(architecture_existed));
+
+		for( t = d->targets; t != NULL ; t = t->next ) {
+			int i;
+
+			if( !t->existed )
+				continue;
+
+			i = atomlist_ofs(&d->architectures,
+					t->architecture_atom);
+			assert( i >= 0 );
+			if( i >= 0 ) {
+				architecture_existed[i] = true;
+				/* only warn about new ones if there
+				 * is at least one old one, otherwise
+				 * it's just a new distribution */
+				have_old = true;
+			}
+		}
+		for( i = 0 ; have_old && i < d->architectures.count ; i++ ) {
+			architecture_t a;
+
+			if( architecture_existed[i] )
+				continue;
+
+			a = d->architectures.atoms[i];
+
+			fprintf(stderr,
+"New architecture '%s' in '%s'. Perhaps you want to call\n"
+"reprepro flood '%s' '%s'\n"
+"to populate it with architecture 'all' packages from other architectures.\n",
+				atoms_architectures[a], d->codename,
+				d->codename, atoms_architectures[a]);
+		}
+
+		/* create databases, so we know next time what is new */
 		for( t = d->targets; t != NULL ; t = t->next ) {
 			if( t->existed )
 				continue;
@@ -523,7 +558,7 @@ static retvalue readversionfile(struct database *db, bool nopackagesyet) {
 	retvalue r;
 	int c;
 
-	versionfilename = calc_dirconcat(db->directory, "version");
+	versionfilename = dbfilename("version");
 	if( versionfilename == NULL )
 		return RET_ERROR_OOM;
 	f = fopen(versionfilename, "r");
@@ -604,7 +639,7 @@ static retvalue readversionfile(struct database *db, bool nopackagesyet) {
 		fprintf(stderr,
 "According to %s/version this database was created with a future version\n"
 "and uses features this version cannot understand. Aborting...\n",
-				db->directory);
+				global.dbdir);
 		return RET_ERROR;
 	}
 
@@ -614,14 +649,14 @@ static retvalue readversionfile(struct database *db, bool nopackagesyet) {
 		fprintf(stderr,
 "According to %s/version this database was created with a yet unsupported\n"
 "database library. Aborting...\n",
-				db->directory);
+				global.dbdir);
 		return RET_ERROR;
 	}
 	if( strncmp(db->lastsupporteddbversion, "bdb", 3) != 0 ) {
 		fprintf(stderr,
 "According to %s/version this database was created with a yet unsupported\n"
 "database library. Aborting...\n",
-				db->directory);
+				global.dbdir);
 		return RET_ERROR;
 	}
 	r = dpkgversions_cmp(LIBDB_VERSION_STRING, db->lastsupporteddbversion, &c);
@@ -632,7 +667,7 @@ static retvalue readversionfile(struct database *db, bool nopackagesyet) {
 "According to %s/version this database was created with a future version\n"
 "%s of libdb. The libdb version this binary is linked against cannot yet\n"
 "handle this format. Aborting...\n",
-				db->directory, db->dbversion+3);
+				global.dbdir, db->dbversion+3);
 		return RET_ERROR;
 	}
 	return RET_OK;
@@ -643,7 +678,7 @@ static retvalue writeversionfile(struct database *db) {
 	FILE *f;
 	int i, e;
 
-	versionfilename = calc_dirconcat(db->directory, "version.new");
+	versionfilename = dbfilename("version.new");
 	if( versionfilename == NULL )
 		return RET_ERROR_OOM;
 	f = fopen(versionfilename, "w");
@@ -711,7 +746,7 @@ static retvalue writeversionfile(struct database *db) {
 		free(versionfilename);
 		return RET_ERRNO(e);
 	}
-	finalversionfilename = calc_dirconcat(db->directory, "version");
+	finalversionfilename = dbfilename("version");
 	if( finalversionfilename == NULL ) {
 		unlink(versionfilename);
 		free(versionfilename);
@@ -759,12 +794,12 @@ static retvalue createnewdatabase(struct database *db, struct distribution *dist
  * - if readonly, do not create but return with RET_NOTHING
  * - lock database, waiting a given amount of time if already locked
  */
-retvalue database_create(struct database **result, const char *dbdir, struct distribution *alldistributions, bool fast, bool nopackages, bool allowunused, bool readonly, size_t waitforlock, bool verbosedb, bool legacymd5format) {
+retvalue database_create(struct database **result, struct distribution *alldistributions, bool fast, bool nopackages, bool allowunused, bool readonly, size_t waitforlock, bool verbosedb, bool legacymd5format) {
 	struct database *n;
 	retvalue r;
 	bool packagesfileexists, trackingfileexists, nopackagesyet;
 
-	if( readonly && !isdir(dbdir) ) {
+	if( readonly && !isdir(global.dbdir) ) {
 		if( verbose >= 0 )
 			fprintf(stderr, "Exiting without doing anything, as there is no database yet that could result in other actions.\n");
 		return RET_NOTHING;
@@ -773,11 +808,6 @@ retvalue database_create(struct database **result, const char *dbdir, struct dis
 	n = calloc(1, sizeof(struct database));
 	if( n == NULL )
 		return RET_ERROR_OOM;
-	n->directory = strdup(dbdir);
-	if( n->directory == NULL ) {
-		free(n);
-		return RET_ERROR_OOM;
-	}
 
 	r = database_lock(n, waitforlock);
 	assert( r != RET_NOTHING );
@@ -1752,14 +1782,13 @@ retvalue database_droppackages(struct database *database, const char *identifier
 	return database_dropsubtable(database, "packages.db", identifier);
 }
 
-retvalue database_openfiles(struct database *db, const char *filesdir) {
+retvalue database_openfiles(struct database *db) {
 	retvalue r;
 	struct strlist identifiers;
 	bool checksumsexisted;
 
 	assert( db->checksums == NULL && db->oldmd5sums == NULL );
 	assert( db->contents == NULL );
-	assert( db->filesdir == NULL );
 
 	r = database_listsubtables(db, "contents.cache.db", &identifiers);
 	if( RET_IS_OK(r) ) {
@@ -1768,25 +1797,19 @@ retvalue database_openfiles(struct database *db, const char *filesdir) {
 "Your %s/contents.cache.db file still contains a table of cached file lists\n"
 "in the old (pre 3.0.0) format. You have to either delete that file (and lose\n"
 "all caches of file lists) or run reprepro with argument translatefilelists\n"
-"to translate the old caches into the new format.\n",	db->directory);
+"to translate the old caches into the new format.\n",	global.dbdir);
 			strlist_done(&identifiers);
 			return RET_ERROR;
 		}
 		strlist_done(&identifiers);
 	}
 
-	db->filesdir = strdup(filesdir);
-	if( db->filesdir == NULL )
-		return RET_ERROR_OOM;
-
 	r = database_hasdatabasefile(db, "checksums.db", &checksumsexisted);
 	r = database_table(db, "checksums.db", "pool",
 			dbt_BTREE, DB_CREATE,
 			&db->checksums);
 	assert( r != RET_NOTHING );
 	if( RET_WAS_ERROR(r) ) {
-		free(db->filesdir);
-		db->filesdir = NULL;
 		db->checksums = NULL;
 		db->oldmd5sums = NULL;
 		return r;
@@ -1800,16 +1823,17 @@ retvalue database_openfiles(struct database *db, const char *filesdir) {
 		db->capabilities.nomd5legacy = true;
 	else if( RET_WAS_ERROR(r) ) {
 		(void)table_close(db->checksums);
-		free(db->filesdir);
-		db->filesdir = NULL;
 		db->checksums = NULL;
 		db->oldmd5sums = NULL;
 		return r;
-	}
+	} else
+		db->capabilities.nomd5legacy = false;
+
 	if( !db->capabilities.nomd5legacy )
 		fprintf(stderr,
 "Warning: database uses deprecated format.\n"
-"Please consider running translatefilelists to update to the new format.\n");
+"Please consider running translatelegacychecksums to update to the new format.\n");
+
 	// TODO: only create this file once it is actually needed...
 	r = database_table(db, "contents.cache.db", "compressedfilelists",
 			dbt_BTREE, DB_CREATE, &db->contents);
@@ -1817,7 +1841,6 @@ retvalue database_openfiles(struct database *db, const char *filesdir) {
 	if( RET_WAS_ERROR(r) ) {
 		(void)table_close(db->checksums);
 		(void)table_close(db->oldmd5sums);
-		db->filesdir = NULL;
 		db->checksums = NULL;
 		db->oldmd5sums = NULL;
 		db->contents = NULL;
@@ -1838,15 +1861,13 @@ retvalue database_openreleasecache(struct database *database, const char *codena
 	 * be very current then.
 	 * */
 
-	oldcachefilename = calc_dirconcat(database_directory(database),
-			"release.cache.db");
+	oldcachefilename = dbfilename("release.cache.db");
 	if( oldcachefilename == NULL )
 		return RET_ERROR_OOM;
 	if( isregularfile(oldcachefilename) ) {
 		char *newcachefilename;
 
-		newcachefilename = calc_dirconcat(database_directory(database),
-				"release.caches.db");
+		newcachefilename = dbfilename("release.caches.db");
 		if( newcachefilename == NULL ) {
 			free(oldcachefilename);
 			return RET_ERROR_OOM;
@@ -1882,11 +1903,6 @@ retvalue database_openreleasecache(struct database *database, const char *codena
 	return r;
 }
 
-/* concat filesdir. return NULL if OutOfMemory */
-char *files_calcfullfilename(const struct database *database,const char *filekey) {
-	return calc_dirconcat(database->filesdir, filekey);
-}
-
 static retvalue table_copy(struct table *oldtable, struct table *newtable) {
 	retvalue r;
 	struct cursor *cursor;
@@ -1919,17 +1935,17 @@ retvalue database_translate_filelists(struct database *database) {
 		if( !strlist_in(&identifiers, "filelists") ) {
 			fprintf(stderr,
 "Your %s/contents.cache.db file does not contain an old style database!\n",
-					database->directory);
+					global.dbdir);
 			strlist_done(&identifiers);
 			return RET_NOTHING;
 		}
 		strlist_done(&identifiers);
 	}
 
-	dbname = calc_dirconcat(database->directory, "contents.cache.db");
+	dbname = dbfilename("contents.cache.db");
 	if( dbname == NULL )
 		return RET_ERROR_OOM;
-	tmpdbname = calc_dirconcat(database->directory, "old.contents.cache.db");
+	tmpdbname = dbfilename("old.contents.cache.db");
 	if( tmpdbname == NULL ) {
 		free(dbname);
 		return RET_ERROR_OOM;
@@ -2002,3 +2018,248 @@ retvalue database_translate_filelists(struct database *database) {
 	free(dbname);
 	return RET_OK;
 }
+
+/* This is already implemented as standalone functions duplicating a bit
+ * of database_create and from files.c,
+ * because database_create is planed to error out if * there is still an old
+ * files.db and files.c is supposed to lose all support for it in the next
+ * major version */
+
+static inline retvalue translate(struct table *oldmd5sums, struct table *newchecksums) {
+	long numold = 0, numnew = 0, numreplace = 0, numretro = 0;
+	struct cursor *cursor, *newcursor;
+	const char *filekey, *md5sum, *all;
+	size_t alllen;
+	retvalue r;
+
+	/* first add all md5sums to checksums if not there yet */
+
+	r = table_newglobalcursor(oldmd5sums, &cursor);
+	if( RET_WAS_ERROR(r) )
+		return r;
+	while( cursor_nexttemp(oldmd5sums, cursor,
+				&filekey, &md5sum) ) {
+		struct checksums *n = NULL;
+		const char *combined;
+		size_t combinedlen;
+
+		r = table_gettemprecord(newchecksums, filekey,
+				&all, &alllen);
+		if( RET_IS_OK(r) )
+			r = checksums_setall(&n, all, alllen, NULL);
+		if( RET_IS_OK(r) ) {
+			if( checksums_matches(n, cs_md5sum, md5sum) ) {
+				/* already there, nothing to do */
+				checksums_free(n);
+				numnew++;
+				continue;
+			}
+			/* new item does not match */
+			if( verbose > 0 )
+				printf(
+"Overwriting stale new-checksums entry '%s'!\n",
+						filekey);
+			numreplace++;
+			checksums_free(n);
+			n = NULL;
+		}
+		if( RET_WAS_ERROR(r) ) {
+			(void)cursor_close(oldmd5sums, cursor);
+			return r;
+		}
+		/* parse and recreate, to only have sanitized strings
+		 * in the database */
+		r = checksums_parse(&n, md5sum);
+		assert( r != RET_NOTHING );
+		if( RET_WAS_ERROR(r) ) {
+			(void)cursor_close(oldmd5sums, cursor);
+			return r;
+		}
+
+		r = checksums_getcombined(n, &combined, &combinedlen);
+		assert( r != RET_NOTHING );
+		if( !RET_IS_OK(r) ) {
+			(void)cursor_close(oldmd5sums, cursor);
+			return r;
+		}
+		numold++;
+		r = table_adduniqsizedrecord(newchecksums, filekey,
+				combined, combinedlen + 1, true, false);
+		assert( r != RET_NOTHING);
+		if( !RET_IS_OK(r) ) {
+			(void)cursor_close(oldmd5sums, cursor);
+			return r;
+		}
+	}
+	r = cursor_close(oldmd5sums, cursor);
+	if( RET_WAS_ERROR(r) )
+		return r;
+
+	/* then delete everything from checksums that is not in md5sums */
+
+	r = table_newglobalcursor(oldmd5sums, &cursor);
+	if( RET_WAS_ERROR(r) )
+		return r;
+	r = table_newglobalcursor(newchecksums, &newcursor);
+	if( RET_WAS_ERROR(r) ) {
+		cursor_close(oldmd5sums, cursor);
+		return r;
+	}
+	while( cursor_nexttemp(oldmd5sums, cursor,
+				&filekey, &md5sum) ) {
+		bool more;
+		int cmp;
+		const char *newfilekey, *dummy;
+
+		do {
+			more = cursor_nexttemp(newchecksums, newcursor,
+				&newfilekey, &dummy);
+			/* should have been added in the last step */
+			assert( more );
+			cmp = strcmp(filekey, newfilekey);
+			/* should have been added in the last step */
+			assert( cmp >= 0 );
+			more = cmp > 0;
+			if( more ) {
+				numretro++;
+				if( verbose > 0 )
+					printf(
+"Deleting stale new-checksums entry '%s'!\n",
+						newfilekey);
+				r = cursor_delete(newchecksums, newcursor,
+						newfilekey, dummy);
+				if( RET_WAS_ERROR(r) ) {
+					cursor_close(oldmd5sums, cursor);
+					cursor_close(newchecksums, newcursor);
+					return r;
+				}
+			}
+		} while( more );
+	}
+	r = cursor_close(oldmd5sums, cursor);
+	if( RET_WAS_ERROR(r) )
+		return r;
+	r = cursor_close(newchecksums, newcursor);
+	if( RET_WAS_ERROR(r) )
+		return r;
+	if( verbose >= 0 ) {
+		printf("%ld packages were already in the new checksums.db\n",
+				numnew);
+		printf("%ld packages were added to the new checksums.db\n",
+				numold - numreplace);
+		if( numretro != 0 )
+			printf(
+"%ld were only in checksums.db and not in files.db\n"
+"This should only have happened if you added them with a newer version\n"
+"and then deleted them with an older version of reprepro.\n",
+				numretro);
+		if( numreplace != 0 )
+			printf(
+"%ld were different checksums.db and not in files.db\n"
+"This should only have happened if you added them with a newer version\n"
+"and then deleted them with an older version of reprepro and\n"
+"then readded them with a old version.\n",
+				numreplace);
+		if( numretro != 0 || numreplace != 0 )
+			printf(
+"If you never run a old version after a new version,\n"
+"you might want to check with check and checkpool if something went wrong.\n");
+	}
+	return RET_OK;
+}
+
+retvalue database_translate_legacy_checksums(bool verbosedb) {
+	struct database *n;
+	struct table *newchecksums, *oldmd5sums;
+	char *fullfilename;
+	retvalue r;
+	int e;
+
+	if( !isdir(global.dbdir) ) {
+		fprintf(stderr, "Cannot find directory '%s'!\n",
+				global.dbdir);
+		return RET_ERROR;
+	}
+
+	n = calloc(1, sizeof(struct database));
+	if( n == NULL )
+		return RET_ERROR_OOM;
+
+	r = database_lock(n, 0);
+	assert( r != RET_NOTHING );
+	if( !RET_IS_OK(r) ) {
+		database_free(n);
+		return r;
+	}
+	n->readonly = READWRITE;
+	n->verbose = verbosedb;
+
+	r = readversionfile(n, false);
+	if( RET_WAS_ERROR(r) ) {
+		releaselock(n);
+		database_free(n);
+		return r;
+	}
+
+	r = database_table(n, "files.db", "md5sums",
+			dbt_BTREE, 0, &oldmd5sums);
+	if( r == RET_NOTHING ) {
+		fprintf(stderr,
+"There is no old files.db in %s. Nothing to translate!\n",
+				global.dbdir);
+		releaselock(n);
+		database_free(n);
+		return RET_NOTHING;
+	} else if( RET_WAS_ERROR(r) ) {
+		releaselock(n);
+		database_free(n);
+		return r;
+	}
+
+	r = database_table(n, "checksums.db", "pool",
+			dbt_BTREE, DB_CREATE,
+			&newchecksums);
+	assert( r != RET_NOTHING );
+	if( RET_WAS_ERROR(r) ) {
+		(void)table_close(oldmd5sums);
+		releaselock(n);
+		database_free(n);
+		return r;
+	}
+
+	r = translate(oldmd5sums, newchecksums);
+	if( RET_WAS_ERROR(r) ) {
+		(void)table_close(oldmd5sums);
+		(void)table_close(newchecksums);
+		releaselock(n);
+		database_free(n);
+		return r;
+	}
+
+	(void)table_close(oldmd5sums);
+	r = table_close(newchecksums);
+	if( RET_WAS_ERROR(r) ) {
+		releaselock(n);
+		database_free(n);
+		return r;
+	}
+	fullfilename = dbfilename("files.db");
+	if( fullfilename == NULL ) {
+		releaselock(n);
+		database_free(n);
+		return RET_ERROR_OOM;
+	}
+	e = deletefile(fullfilename);
+	if( e != 0 ) {
+		fprintf(stderr, "Could not delete '%s'!\n"
+"It can now savely be deleted and it all that is left to be done!\n",
+				fullfilename);
+		database_free(n);
+		return RET_ERRNO(e);
+	}
+	n->capabilities.nomd5legacy = true;
+	r = writeversionfile(n);
+	releaselock(n);
+	database_free(n);
+	return r;
+}
