@@ -9,9 +9,6 @@ struct references;
 struct filesdb;
 
 struct database {
-	char *directory;
-	/* for the files database: */
-	char *filesdir;
 	/*@null@*/ struct table *checksums, *oldmd5sums, *contents;
 	/* for the references database: */
 	/*@null@*/ struct table *references;
