@@ -68,17 +68,21 @@ enum config_option_owner { 	CONFIG_OWNER_DEFAULT=0,
 extern int verbose;
 extern struct global_config {
 	const char *basedir;
+	const char *dbdir;
 	const char *outdir;
 	const char *distdir;
 	const char *confdir;
 	const char *methoddir;
 	const char *logdir;
 	const char *listdir;
+	const char *morguedir;
 	/* deprecated: */
 	const char *overridedir;
 	/* flags: */
 	bool keepdirectories;
 	bool keeptemporaries;
+	/* verbosity of downloading statistics */
+	int showdownloadpercent;
 } global;
 
 enum compression { c_none, c_gzip, c_bzip2, c_lzma, c_COUNT };
