@@ -25,12 +25,8 @@
 #include <string.h>
 #include <malloc.h>
 #include <fcntl.h>
-#ifdef HAVE_LIBGPGME
-#include <gpg-error.h>
-#include <gpgme.h>
-#endif
-#include "globals.h"
-#include "error.h"
+
+#include "signature_p.h"
 #include "mprintf.h"
 #include "strlist.h"
 #include "dirs.h"
@@ -38,15 +34,15 @@
 #include "chunks.h"
 #include "release.h"
 #include "readtextfile.h"
-#include "signature.h"
 
 #ifdef HAVE_LIBGPGME
-static gpgme_ctx_t context = NULL;
+gpgme_ctx_t context = NULL;
 
-static retvalue gpgerror(gpg_error_t err) {
+retvalue gpgerror(gpg_error_t err) {
 	if( err != 0 ) {
-		fprintf(stderr,"gpgme gave %s error: %s\n",
-				gpg_strsource(err), gpg_strerror(err));
+		fprintf(stderr, "gpgme gave error %s:%d:  %s\n",
+				gpg_strsource(err), gpg_err_code(err),
+				gpg_strerror(err));
 		if( gpg_err_code(err) == GPG_ERR_ENOMEM )
 			return RET_ERROR_OOM;
 		else
@@ -77,6 +73,7 @@ retvalue signature_init(bool allowpassphrase){
 #ifdef HAVE_LIBGPGME
 	gpg_error_t err;
 
+	gpgme_check_version(NULL);
 	if( context != NULL )
 		return RET_NOTHING;
 	err = gpgme_engine_check_version(GPGME_PROTOCOL_OpenPGP);
@@ -104,231 +101,6 @@ void signatures_done(void) {
 #endif /* HAVE_LIBGPGME */
 }
 
-#ifdef HAVE_LIBGPGME
-static inline retvalue containskey(const char *key, const char *fingerprint) {
-	size_t fl,kl;
-	const char *keypart,*p;
-
-	fl = strlen(fingerprint);
-
-	keypart = key;
-	while( true ) {
-		while( *keypart != '\0' && xisspace(*keypart) )
-			keypart++;
-		if( *keypart == '\0' )
-			/* nothing more to check, so nothing fulfilled */
-			return RET_NOTHING;
-		p = keypart;
-		while( *p != '\0' && !xisspace(*p) && *p != '|' )
-			p++;
-		kl = p-keypart;
-		if( kl < 8 ) {
-			fprintf(stderr, "Key id too short (less than 8 characters) in '%s'!\n",key);
-			return RET_ERROR;
-		}
-		if( kl < fl && strncasecmp(fingerprint+fl-kl,keypart,kl) == 0 )
-			return RET_OK;
-		keypart = p;
-		while( *keypart != '\0' && xisspace(*keypart) )
-			keypart++;
-		if( *keypart == '\0' )
-			return RET_NOTHING;
-		if( *keypart == '|' )
-			keypart++;
-		else {
-			fprintf(stderr,"Space separated values in keyid '%s'!\n(Use | symbols to separate multiple possible keys!)\n",key);
-			return RET_ERROR;
-		}
-	}
-}
-#endif /* HAVE_LIBGPGME */
-
-retvalue signature_check(const struct strlist *requirements, const char *releasegpg, const char *release) {
-	retvalue r;
-#ifdef HAVE_LIBGPGME
-	gpg_error_t err;
-	int fd,gpgfd;
-	gpgme_data_t dh,dh_gpg;
-	gpgme_verify_result_t result;
-	gpgme_signature_t s;
-	bool totalfulfilled[requirements->count], keyfulfills[requirements->count];
-	int i,j;
-#endif /* HAVE_LIBGPGME */
-
-	assert( requirements->count > 0 );
-
-	if( release == NULL || releasegpg == NULL )
-		return RET_ERROR_OOM;
-
-	r = signature_init(false);
-	if( RET_WAS_ERROR(r) )
-		return r;
-
-#ifdef HAVE_LIBGPGME
-	for( i = 0 ; i < requirements->count ; i++ )
-		totalfulfilled[i] = false;
-	/* Read the file and its signature into memory: */
-	gpgfd = open(releasegpg, O_RDONLY|O_NOCTTY);
-	if( gpgfd < 0 ) {
-		int e = errno;
-		fprintf(stderr, "Error opening '%s': %s\n", releasegpg, strerror(e));
-		return RET_ERRNO(e);
-	}
-	fd = open(release, O_RDONLY|O_NOCTTY);
-	if( fd < 0 ) {
-		int e = errno;
-		(void)close(gpgfd);
-		fprintf(stderr, "Error opening '%s': %s\n", release, strerror(e));
-		return RET_ERRNO(e);
-	}
-	err = gpgme_data_new_from_fd(&dh_gpg, gpgfd);
-	if( err != 0 ) {
-		(void)close(gpgfd); (void)close(fd);
-		return gpgerror(err);
-	}
-	err = gpgme_data_new_from_fd(&dh, fd);
-	if( err != 0 ) {
-		gpgme_data_release(dh_gpg);
-		(void)close(gpgfd); (void)close(fd);
-		return gpgerror(err);
-	}
-
-	/* Verify the signature */
-
-	err = gpgme_op_verify(context,dh_gpg,dh,NULL);
-	gpgme_data_release(dh_gpg);
-	gpgme_data_release(dh);
-	close(gpgfd);close(fd);
-	if( err != 0 )
-		return gpgerror(err);
-
-	result = gpgme_op_verify_result(context);
-	if( result == NULL ) {
-		fprintf(stderr,"Internal error communicating with libgpgme: no result record!\n\n");
-		return RET_ERROR_GPGME;
-	}
-	for( s = result->signatures ; s != NULL ; s = s->next ) {
-		bool keyused = false;
-
-		for( i = 0 ; i < requirements->count ; i++ ) {
-			r = containskey(requirements->values[i], s->fpr);
-			if( RET_WAS_ERROR(r) )
-				return r;
-			if( RET_IS_OK(r) ) {
-				keyused = true;
-				keyfulfills[i] = true;
-			} else
-				keyfulfills[i] = false;
-		}
-		if( !keyused ) {
-			if( gpg_err_code(s->status) == GPG_ERR_NO_ERROR &&
-					verbose > 10 ) {
-				printf("Valid signature with key '%s', but that key is not looked at.\n", s->fpr);
-			}
-			/* there's no use in checking signatures not sufficient */
-			continue;
-		}
-		assert( RET_IS_OK(r) );
-		switch( gpg_err_code(s->status) ) {
-			case GPG_ERR_NO_ERROR:
-				return RET_OK;
-			case GPG_ERR_KEY_EXPIRED:
-				fprintf(stderr,
-"WARNING: valid signature in '%s' with '%s', which has been expired.\n"
-"         as the key was manually specified it is still accepted!\n",
-						releasegpg, s->fpr);
-				for( i = 0 ; i < requirements->count ; i++ ) {
-					if( keyfulfills[i] )
-						totalfulfilled[i] = true;
-				}
-				continue;
-			case GPG_ERR_CERT_REVOKED:
-				fprintf(stderr,
-"WARNING\n"
-"WARNING: valid signature in '%s' with '%s', which has been revoked.\n"
-"WARNING: as the key was manually specified it is still accepted!\n"
-"WARNING\n", releasegpg, s->fpr);
-				for( i = 0 ; i < requirements->count ; i++ ) {
-					if( keyfulfills[i] )
-						totalfulfilled[i] = true;
-				}
-				continue;
-			case GPG_ERR_SIG_EXPIRED:
-				if( verbose > 0 ) {
-					time_t timestamp = s->timestamp,
-					       exp_timestamp = s->exp_timestamp;
-					fprintf(stderr,
-"'%s' has a valid but expired signature with '%s'\n"
-" signature created %s, expired %s\n",
-						releasegpg, s->fpr,
-						ctime(&timestamp),
-						ctime(&exp_timestamp));
-				}
-				// not accepted:
-				continue;
-			case GPG_ERR_BAD_SIGNATURE:
-				if( verbose > 0 ) {
-					fprintf(stderr,
-"WARNING: '%s' has a invalid signature with '%s'\n", releasegpg, s->fpr);
-				}
-				// not accepted:
-				continue;
-			case GPG_ERR_NO_PUBKEY:
-				if( verbose > 0 ) {
-					fprintf(stderr,
-"Could not check validity of signature with '%s' in '%s' as public key missing!\n",
-						s->fpr, releasegpg);
-				}
-				// not accepted:
-				continue;
-			case GPG_ERR_GENERAL:
-				fprintf(stderr,
-"gpgme returned an general error verifing signature with '%s' in '%s'!\n"
-"Try running gpg --verify '%s' '%s' manually for hints what is happening.\n"
-"If this does not print any errors, retry the command causing this message.\n",
-						s->fpr, releasegpg,
-						releasegpg, release);
-				continue;
-			/* there sadly no more is a way to make sure we have
-			 * all possible ones handled */
-			default:
-				break;
-		}
-		fprintf(stderr,
-"Error checking signature (gpgme returned unexpected value %d)!\n"
-"Please file a bug report, so reprepro can handle this in the future.\n",
-			gpg_err_code(s->status));
-		return RET_ERROR_GPGME;
-	}
-	for( i = 0 ; i < requirements->count ; i++ ) {
-		if( totalfulfilled[i] )
-			continue;
-		j = 0;
-		while( j < requirements->count && !totalfulfilled[j] )
-			j++;
-		if( j >= requirements->count ) {
-			/* none of the requirements matches */
-			fprintf(stderr,
-"Error: No accepted signature for '%s' found in '%s'!\n",
-					release, releasegpg);
-		} else {
-			fprintf(stderr,
-"Error: '%s' misses requirement '%s'.\n"
-"At least one other requirement on that file was not missed,\n"
-"but all requirements have to be fulfilled to accept it.\n",
-					releasegpg, requirements->values[i]);
-		}
-		return RET_ERROR_BADSIG;
-	}
-	return RET_OK;
-#else /* HAVE_LIBGPGME */
-	fprintf(stderr,
-"ERROR: Cannot check signature as this reprepro binary is compiled with support\n"
-"for libgpgme.\n"); // TODO: "Only running external programs is supported.\n"
-	return RET_ERROR_GPGME;
-#endif /* HAVE_LIBGPGME */
-}
-
 static retvalue signature_sign(const char *options, const char *filename, const char *signaturename, bool willcleanup) {
 	retvalue r;
 	int ret, e;
@@ -523,46 +295,72 @@ static retvalue signature_sign(const char *options, const char *filename, const
  * which are mentioned in the signature (all). set broken if all signatures
  * was broken (hints to a broken file, as opposed to expired or whatever
  * else may make a signature invalid)). */
-static retvalue checksigs(const char *filename, struct strlist *valid, struct strlist *all, bool *broken) {
+static retvalue checksigs(const char *filename, struct signatures **signatures_p, bool *broken) {
 	gpgme_verify_result_t result;
 	gpgme_signature_t s;
 	bool had_valid = false, had_broken = false;
+	size_t count;
+	struct signatures *signatures;
+	struct signature *sig;
 
 	result = gpgme_op_verify_result(context);
 	if( result == NULL ) {
 		fprintf(stderr,"Internal error communicating with libgpgme: no result record!\n\n");
 		return RET_ERROR_GPGME;
 	}
+	if( signatures_p != NULL ) {
+		count = 0;
+		for( s = result->signatures ; s != NULL ; s = s->next ) {
+			count++;
+		}
+		signatures = calloc(1, sizeof(struct signatures) +
+				count * sizeof(struct signature));
+		if( FAILEDTOALLOC(signatures) )
+			return RET_ERROR_OOM;
+		signatures->count = count;
+		signatures->validcount = 0;
+		sig = signatures->signatures;
+	} else {
+		signatures = NULL;
+		sig = NULL;
+	}
 	for( s = result->signatures ; s != NULL ; s = s->next ) {
-		if( all != NULL ) {
-			retvalue r = strlist_add_dup(all, s->fpr);
-			if( RET_WAS_ERROR(r) )
-				return r;
+		enum signature_state state = sist_error;
+
+		if( signatures_p != NULL ) {
+			sig->keyid = strdup(s->fpr);
+			if( FAILEDTOALLOC(sig->keyid) ) {
+				signatures_free(signatures);
+				return RET_ERROR_OOM;
+			}
 		}
 		switch( gpg_err_code(s->status) ) {
 			case GPG_ERR_NO_ERROR:
 				had_valid = true;
-				if( valid != NULL ) {
-					retvalue r = strlist_add_dup(valid,
-								s->fpr);
-					if( RET_WAS_ERROR(r) )
-						return r;
-				}
-				continue;
+				state = sist_valid;
+				if( signatures )
+					signatures->validcount++;
+				break;
 			case GPG_ERR_KEY_EXPIRED:
 				had_valid = true;
 				if( verbose > 0 )
 					fprintf(stderr,
 "Ignoring signature with '%s' on '%s', as the key has expired.\n",
 						s->fpr, filename);
-				continue;
+				state = sist_mostly;
+				if( sig != NULL )
+					sig->expired_key = true;
+				break;
 			case GPG_ERR_CERT_REVOKED:
 				had_valid = true;
 				if( verbose > 0 )
 					fprintf(stderr,
 "Ignoring signature with '%s' on '%s', as the key is revoked.\n",
 						s->fpr, filename);
-				continue;
+				state = sist_mostly;
+				if( sig != NULL )
+					sig->revoced_key = true;
+				break;
 			case GPG_ERR_SIG_EXPIRED:
 				had_valid = true;
 				if( verbose > 0 ) {
@@ -575,21 +373,26 @@ static retvalue checksigs(const char *filename, struct strlist *valid, struct st
 						ctime(&timestamp),
 						ctime(&exp_timestamp));
 				}
-				continue;
+				state = sist_mostly;
+				if( sig != NULL )
+					sig->expired_signature = true;
+				break;
 			case GPG_ERR_BAD_SIGNATURE:
 				had_broken = true;
 				if( verbose > 0 ) {
 					fprintf(stderr,
 "WARNING: '%s' has a invalid signature with '%s'\n", filename, s->fpr);
 				}
-				continue;
+				state = sist_bad;
+				break;
 			case GPG_ERR_NO_PUBKEY:
 				if( verbose > 0 ) {
 					fprintf(stderr,
 "Could not check validity of signature with '%s' in '%s' as public key missing!\n",
 						s->fpr, filename);
 				}
-				continue;
+				state = sist_missing;
+				break;
 			case GPG_ERR_GENERAL:
 				fprintf(stderr,
 "gpgme returned an general error verifing signature with '%s' in '%s'!\n"
@@ -597,20 +400,84 @@ static retvalue checksigs(const char *filename, struct strlist *valid, struct st
 "If this does not print any errors, retry the command causing this message.\n",
 						s->fpr, filename,
 						filename);
-				continue;
+				signatures_free(signatures);
+				return RET_ERROR_GPGME;
 			/* there sadly no more is a way to make sure we have
 			 * all possible ones handled */
 			default:
 				break;
 		}
-		fprintf(stderr,
+		if( state == sist_error ) {
+			fprintf(stderr,
 "Error checking signature (gpgme returned unexpected value %d)!\n"
 "Please file a bug report, so reprepro can handle this in the future.\n",
-			gpg_err_code(s->status));
-		return RET_ERROR_GPGME;
+				gpg_err_code(s->status));
+			signatures_free(signatures);
+			return RET_ERROR_GPGME;
+		}
+		if( sig != NULL ) {
+			sig->state = state;
+			sig++;
+		}
 	}
 	if( broken != NULL && had_broken && ! had_valid )
 		*broken = true;
+	if( signatures_p != NULL )
+		*signatures_p = signatures;
+	return RET_OK;
+}
+
+static retvalue check_primary_keys(struct signatures *signatures) {
+	/* Get the primary keys belonging to each signing key.
+	   This might also invalidate a signature previously believed
+	   valid if the primary key is expired */
+	int i;
+
+	for( i = 0 ; i < signatures->count ; i++ ) {
+		gpg_error_t err;
+		gpgme_key_t gpgme_key = NULL;
+		gpgme_subkey_t subkey;
+		struct signature *sig = &signatures->signatures[i];
+
+		if( sig->state == sist_error || sig->state == sist_missing ) {
+			sig->primary_keyid = strdup(sig->keyid);
+			if( FAILEDTOALLOC(sig->primary_keyid) )
+				return RET_ERROR_OOM;
+			continue;
+		}
+
+		err = gpgme_get_key(context, sig->keyid, &gpgme_key, 0);
+		if( err != 0 ) {
+			fprintf(stderr, "gpgme error %s:%d retrieving key '%s': %s\n",
+					gpg_strsource(err), (int)gpg_err_code(err),
+					sig->keyid, gpg_strerror(err));
+			if( gpg_err_code(err) == GPG_ERR_ENOMEM )
+				return RET_ERROR_OOM;
+			else
+				return RET_ERROR_GPGME;
+		}
+		assert( gpgme_key != NULL );
+		/* the first "sub"key is the primary key */
+		subkey = gpgme_key->subkeys;
+		if( subkey->revoked ) {
+			sig->revoced_key = true;
+			if( sig->state == sist_valid ) {
+				sig->state = sist_mostly;
+				signatures->validcount--;
+			}
+		}
+		if( subkey->expired ) {
+			sig->expired_key = true;
+			if( sig->state == sist_valid ) {
+				sig->state = sist_mostly;
+				signatures->validcount--;
+			}
+		}
+		sig->primary_keyid = strdup(subkey->keyid);
+		gpgme_key_unref(gpgme_key);
+		if( FAILEDTOALLOC(sig->primary_keyid) )
+			return RET_ERROR_OOM;
+	}
 	return RET_OK;
 }
 #endif /* HAVE_LIBGPGME */
@@ -649,8 +516,21 @@ static inline void extractchunk(const char *buffer, const char **begin, const ch
 	*next = afterchanges;
 }
 
+void signatures_free(struct signatures *signatures) {
+	int i;
+
+	if( signatures == NULL )
+		return;
+
+	for( i = 0 ; i < signatures->count ; i++ ) {
+		free(signatures->signatures[i].keyid);
+		free(signatures->signatures[i].primary_keyid);
+	}
+	free(signatures);
+}
+
 #ifdef HAVE_LIBGPGME
-static retvalue extract_signed_data(const char *buffer, size_t bufferlen, const char *filenametoshow, char **chunkread, /*@null@*/ /*@out@*/struct strlist *validkeys, /*@null@*/ /*@out@*/ struct strlist *allkeys, bool *brokensignature, bool *failed) {
+static retvalue extract_signed_data(const char *buffer, size_t bufferlen, const char *filenametoshow, char **chunkread, /*@null@*/ /*@out@*/struct signatures **signatures_p, bool *brokensignature, bool *failed) {
 	const char *startofchanges,*endofchanges,*afterchanges;
 	char *chunk;
 	gpg_error_t err;
@@ -658,12 +538,9 @@ static retvalue extract_signed_data(const char *buffer, size_t bufferlen, const
 	size_t plain_len;
 	char *plain_data;
 	retvalue r;
-	struct strlist validfingerprints, allfingerprints;
+	struct signatures *signatures = NULL;
 	bool foundbroken = false;
 
-	strlist_init(&validfingerprints);
-	strlist_init(&allfingerprints);
-
 	r = signature_init(false);
 	if( RET_WAS_ERROR(r) )
 		return r;
@@ -690,10 +567,9 @@ static retvalue extract_signed_data(const char *buffer, size_t bufferlen, const
 			gpgme_data_release(dh);
 			return gpgerror(err);
 		}
-		if( validkeys != NULL || allkeys != NULL || brokensignature != NULL ) {
+		if( signatures_p != NULL || brokensignature != NULL ) {
 			r = checksigs(filenametoshow,
-				(validkeys!=NULL)?&validfingerprints:NULL,
-				(allkeys!=NULL)?&allfingerprints:NULL,
+				(signatures_p!=NULL)?&signatures:NULL,
 				(brokensignature!=NULL)?&foundbroken:NULL);
 			if( RET_WAS_ERROR(r) ) {
 				gpgme_data_release(dh_gpg);
@@ -713,8 +589,16 @@ static retvalue extract_signed_data(const char *buffer, size_t bufferlen, const
 "Continuing extracting it ignoring all signatures...",
 					filenametoshow, filenametoshow);
 			*failed = true;
+			signatures_free(signatures);
 			return RET_NOTHING;
 		}
+		if( signatures != NULL ) {
+			r = check_primary_keys(signatures);
+			if( RET_WAS_ERROR(r) ) {
+				signatures_free(signatures);
+				return r;
+			}
+		}
 	}
 
 	if( plain_data == NULL )
@@ -772,24 +656,19 @@ static retvalue extract_signed_data(const char *buffer, size_t bufferlen, const
 	free(plain_data);
 #endif
 	if( RET_IS_OK(r) ) {
-		if( validkeys != NULL )
-			strlist_move(validkeys, &validfingerprints);
-		if( allkeys != NULL )
-			strlist_move(allkeys, &allfingerprints);
+		if( signatures_p != NULL )
+			*signatures_p = signatures;
 		if( brokensignature != NULL )
 			*brokensignature = foundbroken;
 	} else {
-		if( validkeys != NULL )
-			strlist_done(&validfingerprints);
-		if( allkeys != NULL )
-			strlist_done(&allfingerprints);
+		signatures_free(signatures);
 	}
 	return r;
 }
 #endif /* HAVE_LIBGPGME */
 
 /* Read a single chunk from a file, that may be signed. */
-retvalue signature_readsignedchunk(const char *filename, const char *filenametoshow, char **chunkread, /*@null@*/ /*@out@*/struct strlist *validkeys, /*@null@*/ /*@out@*/ struct strlist *allkeys, bool *brokensignature) {
+retvalue signature_readsignedchunk(const char *filename, const char *filenametoshow, char **chunkread, /*@null@*/ /*@out@*/struct signatures **signatures_p, bool *brokensignature) {
 	char *chunk, *h, *afterchunk;
 	const char *startofchanges,*endofchanges,*afterchanges;
 	size_t chunklen, len;
@@ -826,10 +705,8 @@ retvalue signature_readsignedchunk(const char *filename, const char *filenametos
 		if( h != NULL )
 			chunk = h;
 		*chunkread = chunk;
-		if( validkeys != NULL )
-			strlist_init(validkeys);
-		if( allkeys != NULL )
-			strlist_init(allkeys);
+		if( signatures_p != NULL )
+			*signatures_p = NULL;
 		if( brokensignature != NULL )
 			*brokensignature = false;
 		return RET_OK;
@@ -843,7 +720,7 @@ retvalue signature_readsignedchunk(const char *filename, const char *filenametos
 
 #ifdef HAVE_LIBGPGME
 	r = extract_signed_data(chunk, chunklen, filenametoshow, chunkread,
-			validkeys, allkeys, brokensignature, &failed);
+			signatures_p, brokensignature, &failed);
 	if( r != RET_NOTHING ) {
 		free(chunk);
 		return r;
@@ -912,11 +789,16 @@ retvalue signature_readsignedchunk(const char *filename, const char *filenametos
 	h = realloc(chunk, len + 1);
 	if( h != NULL )
 		chunk = h;
+	if( signatures_p != NULL ) {
+		/* pointer to structure with count 0 to make clear
+		 * it is not unsigned */
+		*signatures_p = calloc(1, sizeof(struct signatures));
+		if( FAILEDTOALLOC(*signatures_p) ) {
+			free(chunk);
+			return RET_ERROR_OOM;
+		}
+	}
 	*chunkread = chunk;
-	if( validkeys != NULL )
-		strlist_init(validkeys);
-	if( allkeys != NULL )
-		strlist_init(allkeys);
 	if( brokensignature != NULL )
 		*brokensignature = false;
 	return RET_OK;
