@@ -4,6 +4,7 @@ set -e
 
 source $(dirname $0)/test.inc
 
+runtest flood
 runtest exporthooks
 runtest updatecorners
 runtest packagediff
@@ -20,6 +21,9 @@ runtest snapshotcopyrestore
 runtest various1
 runtest various2
 runtest various3
+runtest copy
+runtest buildneeding
+runtest morgue
 
 set +v +x
 echo
