@@ -1,5 +1,5 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2006,2007,2008 Bernhard R. Link
+ *  Copyright (C) 2006,2007,2008,2009 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
@@ -48,8 +48,8 @@ bool interrupted(void) {return false;}
 static void about(bool help) NORETURN;
 static void about(bool help) {
 	fprintf(help?stdout:stderr,
-"modifychanges: Modify a Debian style .changes file\n"
-"Syntax: modifychanges [--create] <changesfile> <commands>\n"
+"changestool: Modify a Debian style .changes file\n"
+"Syntax: changestool [--create] <changesfile> <commands>\n"
 "Possible commands include:\n"
 " verify\n"
 " updatechecksums [<files to update>]\n"
@@ -59,7 +59,6 @@ static void about(bool help) {
 " addrawfile <filenames>\n"
 " add <filenames processed by filename suffix>\n"
 " setdistribution <distributions to list>\n"
-//" create <.dsc and .deb files to include>\n"
 );
 	if( help )
 		exit(EXIT_SUCCESS);
@@ -117,7 +116,6 @@ struct dscfile {
 	struct strlist binaries;
 	char *maintainer;
 	char *controlchunk;
-	struct strlist validkeys,keys;
 	// hard to get:
 	char *section, *priority;
 	// TODO: check Architectures?
@@ -134,8 +132,6 @@ static void dscfile_free(struct dscfile *p) {
 	free(p->version);
 	free(p->maintainer);
 	free(p->controlchunk);
-	strlist_done(&p->keys);
-	strlist_done(&p->validkeys);
 	free(p->section);
 	free(p->priority);
 	checksumsarray_done(&p->expected);
@@ -599,7 +595,7 @@ static retvalue read_dscfile(const char *fullfilename, struct dscfile **dsc) {
 	if( n == NULL )
 		return RET_ERROR_OOM;
 	r = signature_readsignedchunk(fullfilename, fullfilename,
-			&n->controlchunk, &n->validkeys, &n->keys, NULL);
+			&n->controlchunk, NULL, NULL);
 	assert( r != RET_NOTHING );
 	// TODO: can this be ignored sometimes?
 	if( RET_WAS_ERROR(r) ) {
@@ -1026,7 +1022,7 @@ static retvalue parse_changes(const char *changesfile, const char *chunk, struct
 #define CHANGES_WRITE_DISTRIBUTIONS 	0x40
 #define CHANGES_WRITE_ALL 	      0xFFFF
 
-static retvalue write_changes_file(const char *changesfilename,struct changes *c, unsigned int flags) {
+static retvalue write_changes_file(const char *changesfilename,struct changes *c, unsigned int flags, bool fakefields) {
 	struct chunkeditfield *cef;
 	char datebuffer[100];
 	retvalue r;
@@ -1089,9 +1085,17 @@ static retvalue write_changes_file(const char *changesfilename,struct changes *c
 		if( cef == NULL )
 			return RET_ERROR_OOM;
 	}
-	cef = cef_newfield("Changes", CEF_KEEP, CEF_LATE, 0, cef);
-	if( cef == NULL )
+	if( fakefields ) {
+		cef = cef_newfield("Changes", CEF_ADDMISSED, CEF_LATE, 0, cef);
+		if( cef == NULL ) {
 			return RET_ERROR_OOM;
+		}
+		cef_setdata(cef, "\n Changes information missing, as not an original .changes file");
+	} else {
+		cef = cef_newfield("Changes", CEF_KEEP, CEF_LATE, 0, cef);
+		if( cef == NULL )
+			return RET_ERROR_OOM;
+	}
 	cef = cef_newfield("Closes", CEF_KEEP, CEF_LATE, 0, cef);
 	if( cef == NULL )
 			return RET_ERROR_OOM;
@@ -1130,9 +1134,17 @@ static retvalue write_changes_file(const char *changesfilename,struct changes *c
 		if( cef == NULL )
 			return RET_ERROR_OOM;
 	}
-	cef = cef_newfield("Urgency", CEF_KEEP, CEF_EARLY, 0, cef);
-	if( cef == NULL )
+	if( fakefields ) {
+		cef = cef_newfield("Urgency", CEF_ADDMISSED, CEF_EARLY, 0, cef);
+		if( cef == NULL ) {
 			return RET_ERROR_OOM;
+		}
+		cef_setdata(cef, "low");
+	} else {
+		cef = cef_newfield("Urgency", CEF_KEEP, CEF_EARLY, 0, cef);
+		if( cef == NULL )
+			return RET_ERROR_OOM;
+	}
 	cef = cef_newfield("Distribution", CEF_KEEP, CEF_EARLY, 0, cef);
 	if( cef == NULL )
 			return RET_ERROR_OOM;
@@ -2022,9 +2034,11 @@ static retvalue updatechecksums(const char *changesfilename, struct changes *c,
 		c->modified = true;
 	}
 	if( c->modified ) {
-		return write_changes_file(changesfilename, c, CHANGES_WRITE_FILES);
+		return write_changes_file(changesfilename, c,
+				CHANGES_WRITE_FILES, false);
 	} else if( improvedchecksum_supported(c, improvedfilehashes) ) {
-		return write_changes_file(changesfilename, c, CHANGES_WRITE_FILES);
+		return write_changes_file(changesfilename, c,
+				CHANGES_WRITE_FILES, false);
 	} else
 		return RET_NOTHING;
 }
@@ -2082,7 +2096,8 @@ static retvalue includeallsources(const char *changesfilename, struct changes *c
 		}
 	}
 	if( c->modified ) {
-		return write_changes_file(changesfilename, c, CHANGES_WRITE_FILES);
+		return write_changes_file(changesfilename, c,
+				CHANGES_WRITE_FILES, false);
 	} else
 		return RET_NOTHING;
 }
@@ -2310,7 +2325,7 @@ static retvalue adddsc(struct changes *c, const char *dscfilename, const struct
 	return RET_OK;
 }
 
-static retvalue adddscs(const char *changesfilename, struct changes *c, int argc, char **argv, const struct strlist *searchpath) {
+static retvalue adddscs(const char *changesfilename, struct changes *c, int argc, char **argv, const struct strlist *searchpath, bool fakefields) {
 	if( argc <= 0 ) {
 		fprintf(stderr, "Filenames of .dsc files to include expected!\n");
 		return RET_ERROR;
@@ -2323,7 +2338,7 @@ static retvalue adddscs(const char *changesfilename, struct changes *c, int argc
 	}
 	if( c->modified ) {
 		return write_changes_file(changesfilename, c,
-				CHANGES_WRITE_ALL);
+				CHANGES_WRITE_ALL, fakefields);
 	} else
 		return RET_NOTHING;
 }
@@ -2504,7 +2519,7 @@ static retvalue adddeb(struct changes *c, const char *debfilename, const struct
 	return RET_OK;
 }
 
-static retvalue adddebs(const char *changesfilename, struct changes *c, int argc, char **argv, const struct strlist *searchpath) {
+static retvalue adddebs(const char *changesfilename, struct changes *c, int argc, char **argv, const struct strlist *searchpath, bool fakefields) {
 	if( argc <= 0 ) {
 		fprintf(stderr, "Filenames of .deb files to include expected!\n");
 		return RET_ERROR;
@@ -2517,7 +2532,7 @@ static retvalue adddebs(const char *changesfilename, struct changes *c, int argc
 	}
 	if( c->modified ) {
 		return write_changes_file(changesfilename, c,
-				CHANGES_WRITE_ALL);
+				CHANGES_WRITE_ALL, fakefields);
 	} else
 		return RET_NOTHING;
 }
@@ -2596,7 +2611,7 @@ static retvalue addrawfile(struct changes *c, const char *filename, const struct
 	return RET_OK;
 }
 
-static retvalue addrawfiles(const char *changesfilename, struct changes *c, int argc, char **argv, const struct strlist *searchpath) {
+static retvalue addrawfiles(const char *changesfilename, struct changes *c, int argc, char **argv, const struct strlist *searchpath, bool fakefields) {
 	if( argc <= 0 ) {
 		fprintf(stderr, "Filenames of files to add (without further parsing) expected!\n");
 		return RET_ERROR;
@@ -2609,12 +2624,12 @@ static retvalue addrawfiles(const char *changesfilename, struct changes *c, int
 	}
 	if( c->modified ) {
 		return write_changes_file(changesfilename, c,
-				CHANGES_WRITE_FILES);
+				CHANGES_WRITE_FILES, fakefields);
 	} else
 		return RET_NOTHING;
 }
 
-static retvalue addfiles(const char *changesfilename, struct changes *c, int argc, char **argv, const struct strlist *searchpath) {
+static retvalue addfiles(const char *changesfilename, struct changes *c, int argc, char **argv, const struct strlist *searchpath, bool fakefields) {
 	if( argc <= 0 ) {
 		fprintf(stderr, "Filenames of files to add expected!\n");
 		return RET_ERROR;
@@ -2637,7 +2652,7 @@ static retvalue addfiles(const char *changesfilename, struct changes *c, int arg
 	}
 	if( c->modified ) {
 		return write_changes_file(changesfilename, c,
-				CHANGES_WRITE_ALL);
+				CHANGES_WRITE_ALL, fakefields);
 	} else
 		return RET_NOTHING;
 }
@@ -2664,10 +2679,10 @@ static retvalue setdistribution(const char *changesfilename, struct changes *c,
 	strlist_done(&c->distributions);
 	strlist_move(&c->distributions, &distributions);
 	return write_changes_file(changesfilename, c,
-			CHANGES_WRITE_DISTRIBUTIONS);
+			CHANGES_WRITE_DISTRIBUTIONS, false);
 }
 
-static int execute_command(int argc, char **argv, const char *changesfilename, const struct strlist *searchpath, bool file_exists, bool create_file, struct changes *changesdata) {
+static int execute_command(int argc, char **argv, const char *changesfilename, const struct strlist *searchpath, bool file_exists, bool create_file, bool fakefields, struct changes *changesdata) {
 	const char *command = argv[0];
 	retvalue r;
 
@@ -2703,7 +2718,7 @@ static int execute_command(int argc, char **argv, const char *changesfilename, c
 	} else if( strcasecmp(command, "addrawfile") == 0 ) {
 		if( file_exists || create_file )
 			r = addrawfiles(changesfilename, changesdata,
-					argc-1, argv+1, searchpath);
+					argc-1, argv+1, searchpath, fakefields);
 		else {
 			fprintf(stderr, "No such file '%s'!\n",
 					changesfilename);
@@ -2712,7 +2727,7 @@ static int execute_command(int argc, char **argv, const char *changesfilename, c
 	} else if( strcasecmp(command, "adddsc") == 0 ) {
 		if( file_exists || create_file )
 			r = adddscs(changesfilename, changesdata,
-					argc-1, argv+1, searchpath);
+					argc-1, argv+1, searchpath, fakefields);
 		else {
 			fprintf(stderr, "No such file '%s'!\n",
 					changesfilename);
@@ -2721,7 +2736,7 @@ static int execute_command(int argc, char **argv, const char *changesfilename, c
 	} else if( strcasecmp(command, "adddeb") == 0 ) {
 		if( file_exists || create_file )
 			r = adddebs(changesfilename, changesdata,
-					argc-1, argv+1, searchpath);
+					argc-1, argv+1, searchpath, fakefields);
 		else {
 			fprintf(stderr, "No such file '%s'!\n",
 					changesfilename);
@@ -2730,7 +2745,7 @@ static int execute_command(int argc, char **argv, const char *changesfilename, c
 	} else if( strcasecmp(command, "add") == 0 ) {
 		if( file_exists || create_file )
 			r = addfiles(changesfilename, changesdata,
-					argc-1, argv+1, searchpath);
+					argc-1, argv+1, searchpath, fakefields);
 		else {
 			fprintf(stderr, "No such file '%s'!\n",
 					changesfilename);
@@ -2776,6 +2791,7 @@ int main(int argc,char *argv[]) {
 	static const struct option longopts[] = {
 		{"help", no_argument, NULL, 'h'},
 		{"create", no_argument, NULL, 'C'},
+		{"create-with-all-fields", no_argument, &longoption, 4},
 		{"searchpath", required_argument, NULL, 's'},
 		{"gunzip", required_argument, &longoption, 1},
 		{"bunzip2", required_argument, &longoption, 2},
@@ -2786,7 +2802,7 @@ int main(int argc,char *argv[]) {
 	const char *changesfilename;
 	bool file_exists;
 	bool create_file = false;
-	struct strlist validkeys,keys;
+	bool all_fields = false;
 	struct strlist searchpath;
 	struct changes *changesdata IFSTUPIDCC(=NULL);
 	char *gunzip = NULL, *bunzip2 = NULL, *unlzma = NULL;
@@ -2807,6 +2823,10 @@ int main(int argc,char *argv[]) {
 					case 3:
 						unlzma = strdup(optarg);
 						break;
+					case 4:
+						create_file = true;
+						all_fields = true;
+						break;
 				}
 				break;
 			case 'h':
@@ -2840,7 +2860,7 @@ int main(int argc,char *argv[]) {
 		char *changes;
 
 		r = signature_readsignedchunk(changesfilename, changesfilename,
-				&changes, &validkeys, &keys, NULL);
+				&changes, NULL, NULL);
 		if( !RET_IS_OK(r) ) {
 			signatures_done();
 			if( r == RET_ERROR_OOM )
@@ -2855,8 +2875,6 @@ int main(int argc,char *argv[]) {
 			changesdata = NULL;
 		}
 	} else {
-		strlist_init(&keys);
-		strlist_init(&validkeys);
 		changesdata = calloc(1,sizeof(struct changes));
 		if( changesdata != NULL )
 			changesdata->filename = strdup(changesfilename);
@@ -2872,10 +2890,10 @@ int main(int argc,char *argv[]) {
 		argc -= (optind+1);
 		argv += (optind+1);
 		r = execute_command(argc, argv, changesfilename, &searchpath,
-		                    file_exists, create_file, changesdata);
+		                    file_exists, create_file, all_fields,
+				    changesdata);
 	}
 	changes_free(changesdata);
-	strlist_done(&keys);
 
 	signatures_done();
 	if( RET_IS_OK(r) )
