@@ -50,6 +50,7 @@
 #include <errno.h>
 #include <assert.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/wait.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -58,6 +59,7 @@
 #include <malloc.h>
 #include <string.h>
 #include <ctype.h>
+#include <fcntl.h>
 #include "error.h"
 #include "ignore.h"
 #include "mprintf.h"
@@ -71,7 +73,8 @@
 #include "updates.h"
 #include "upgradelist.h"
 #include "distribution.h"
-#include "terms.h"
+#include "tracking.h"
+#include "termdecide.h"
 #include "filterlist.h"
 #include "readrelease.h"
 #include "log.h"
@@ -158,6 +161,7 @@ struct update_pattern {
 	struct filterlist filterlist;
 	// NULL means nothing to execute after lists are downloaded...
 	/*@null@*/char *listhook;
+	/*@null@*/char *shellhook;
 	/* checksums to not read check in Release file: */
 	bool ignorehashes[cs_hashCOUNT];
 	/* the name of the flat component, causing flat mode if non-NULL*/
@@ -254,6 +258,7 @@ static void update_pattern_free(/*@only@*/struct update_pattern *update) {
 	term_free(update->includecondition);
 	filterlist_release(&update->filterlist);
 	free(update->listhook);
+	free(update->shellhook);
 	remote_repository_free(update->repository);
 	free(update);
 }
@@ -339,10 +344,11 @@ CFvalueSETPROC(update_pattern, from)
 CFurlSETPROC(update_pattern, method)
 CFurlSETPROC(update_pattern, fallback)
 /* what here? */
-CFvalueSETPROC(update_pattern, verifyrelease)
+CFallSETPROC(update_pattern, verifyrelease)
 CFlinelistSETPROC(update_pattern, config)
 CFtruthSETPROC(update_pattern, ignorerelease)
 CFscriptSETPROC(update_pattern, listhook)
+CFallSETPROC(update_pattern, shellhook)
 CFfilterlistSETPROC(update_pattern, filterlist)
 CFtermSSETPROC(update_pattern, includecondition)
 
@@ -518,6 +524,7 @@ static const struct configfield updateconfigfields[] = {
 	CF("IgnoreHashes", update_pattern, ignorehashes),
 	CF("VerifyRelease", update_pattern, verifyrelease),
 	CF("ListHook", update_pattern, listhook),
+	CF("ListShellHook", update_pattern, shellhook),
 	CF("FilterFormula", update_pattern, includecondition),
 	CF("FilterList", update_pattern, filterlist),
 	CF("DownloadListsAs", update_pattern, downloadlistsas)
@@ -578,6 +585,13 @@ CFfinishparse(update_pattern) {
 				config_line(iter), n->suite_from);
 			return RET_ERROR;
 		}
+		if( n->listhook != NULL && n->shellhook != NULL ) {
+			fprintf(stderr,
+"%s:%u to %u: Only one of ListHook and ListShellHook allowed per update rule\n",
+				config_filename(iter), config_firstline(iter),
+				config_line(iter));
+			return RET_ERROR;
+		}
 	}
 	return linkedlistfinish(privdata_update_pattern, thisdata_update_pattern,
 			lastdata_p_update_pattern, complete, iter);
@@ -736,6 +750,7 @@ static retvalue instance_pattern(struct update_pattern *pattern, const struct di
 	/*@dependant@*/struct update_pattern *declaration, *p, *listscomponents;
 	bool ignorehashes[cs_hashCOUNT], ignorerelease;
 	const char *verifyrelease;
+	retvalue r;
 
 	update = calloc(1,sizeof(struct update_origin));
 	if( update == NULL )
@@ -830,13 +845,13 @@ static retvalue instance_pattern(struct update_pattern *pattern, const struct di
 		updates_freeorigins(update);
 		return RET_ERROR;
 	}
-	update->from = remote_distribution_prepare(declaration->repository,
+	r = remote_distribution_prepare(declaration->repository,
 			update->suite_from, ignorerelease,
 			verifyrelease, update->flat,
-			ignorehashes);
-	if( FAILEDTOALLOC(update->from) ) {
+			ignorehashes, &update->from);
+	if( RET_WAS_ERROR(r) ) {
 		updates_freeorigins(update);
-		return RET_ERROR_OOM;
+		return r;
 	}
 
 	*origins = update;
@@ -1232,8 +1247,10 @@ retvalue updates_calcindices(struct update_pattern *patterns, struct distributio
 		u_ds = u_d;
 
 		r = getorigins(u_d);
-		if( RET_WAS_ERROR(r) )
+		if( RET_WAS_ERROR(r) ) {
+			result = r;
 			break;
+		}
 		if( RET_IS_OK(r) ) {
 			/* Check if we got all: */
 			r = findmissingupdate(distribution, u_d->origins);
@@ -1280,9 +1297,6 @@ static retvalue updates_startup(struct aptmethodrun *run, struct update_distribu
 			if( verbose >= 0 )
 				fprintf(stderr,"Warning: Override-Files of '%s' ignored as not yet supported while updating!\n",d->distribution->codename);
 		}
-		if( d->distribution->tracking != dt_NONE ) {
-			fprintf(stderr,"WARNING: Updating does not update trackingdata. Trackingdata of %s will be outdated!\n",d->distribution->codename);
-		}
 	}
 	return remote_startup(run);
 }
@@ -1302,11 +1316,10 @@ static retvalue updates_startup(struct aptmethodrun *run, struct update_distribu
  * Step 8: call possible list hooks allowing them to modify the lists       *
  ****************************************************************************/
 
-static retvalue calllisthook(struct update_target *ut, struct update_index_connector *f) {
+static retvalue calllisthook(struct update_target *ut, struct update_index_connector *f, const char *listhook) {
 	struct update_origin *origin = f->origin;
 	const char *oldfilename = remote_index_file(f->remote);
 	const char *oldbasefilename = remote_index_basefile(f->remote);
-	const char *listhook = origin->pattern->listhook;
 	char *newfilename;
 	pid_t child, c;
 	int status;
@@ -1383,6 +1396,121 @@ static retvalue calllisthook(struct update_target *ut, struct update_index_conne
 	return RET_OK;
 }
 
+static retvalue callshellhook(struct update_target *ut, struct update_index_connector *f, const char *shellhook) {
+	struct update_origin *origin = f->origin;
+	const char *oldfilename = remote_index_file(f->remote);
+	const char *oldbasefilename = remote_index_basefile(f->remote);
+	char *newfilename;
+	pid_t child, c;
+	int status;
+	int infd, outfd;
+
+	/* distribution, component, architecture and pattern specific... */
+	newfilename = genlistsfilename(oldbasefilename, 5, "",
+			ut->target->codename,
+			atoms_components[ut->target->component_atom],
+			atoms_architectures[ut->target->architecture_atom],
+			origin->pattern->name, ENDOFARGUMENTS);
+	if( FAILEDTOALLOC(newfilename) )
+		return RET_ERROR_OOM;
+	infd = open(oldfilename, O_RDONLY|O_NOCTTY|O_NOFOLLOW);
+	if( infd < 0 ) {
+		int e = errno;
+
+		fprintf(stderr,
+"Error %d opening expected file '%s': %s!\n"
+"Something strange must go on!\n", e, oldfilename, strerror(e));
+		return RET_ERRNO(e);
+	}
+	(void)unlink(newfilename);
+	outfd = open(newfilename,
+			O_WRONLY|O_NOCTTY|O_NOFOLLOW|O_CREAT|O_EXCL, 0666);
+	if( outfd < 0 ) {
+		int e = errno;
+
+		fprintf(stderr, "Error %d creating '%s': %s!\n", e,
+				newfilename, strerror(e));
+		close(infd);
+		return RET_ERRNO(e);
+	}
+	child = fork();
+	if( child < 0 ) {
+		int e = errno;
+		free(newfilename);
+		fprintf(stderr, "Error %d while forking for shell hook: %s\n",
+				e, strerror(e));
+		(void)close(infd);
+		(void)close(outfd);
+		(void)unlink(newfilename);
+		return RET_ERRNO(e);
+	}
+	if( child == 0 ) {
+		int e;
+
+		assert( dup2(infd, 0) == 0 );
+		assert( dup2(outfd, 1) == 1 );
+		close(infd);
+		close(outfd);
+		(void)closefrom(3);
+		setenv("REPREPRO_BASE_DIR", global.basedir, true);
+		setenv("REPREPRO_OUT_DIR", global.outdir, true);
+		setenv("REPREPRO_CONF_DIR", global.confdir, true);
+		setenv("REPREPRO_DIST_DIR", global.distdir, true);
+		setenv("REPREPRO_LOG_DIR", global.logdir, true);
+		setenv("REPREPRO_FILTER_CODENAME",
+				ut->target->codename, true);
+		setenv("REPREPRO_FILTER_PACKAGETYPE",
+				atoms_architectures[ut->target->packagetype_atom],
+				true);
+		setenv("REPREPRO_FILTER_COMPONENT",
+				atoms_components[ut->target->component_atom],
+				true);
+		setenv("REPREPRO_FILTER_ARCHITECTURE",
+				atoms_architectures[ut->target->architecture_atom],
+				true);
+		setenv("REPREPRO_FILTER_PATTERN", origin->pattern->name, true);
+		execlp("sh", "sh", "-c", shellhook, ENDOFARGUMENTS);
+		e = errno;
+		fprintf(stderr, "Error %d while executing sh -c '%s': %s\n",
+				e, shellhook, strerror(e));
+		exit(255);
+	}
+	(void)close(infd);
+	(void)close(outfd);
+	if( verbose > 5 )
+		fprintf(stderr, "Called sh -c '%s' <'%s' >'%s'\n", shellhook,
+				oldfilename, newfilename);
+	f->afterhookfilename = newfilename;
+	do {
+		c = waitpid(child, &status, WUNTRACED);
+		if( c < 0 ) {
+			int e = errno;
+			fprintf(stderr,
+"Error %d while waiting for shell hook '%s' to finish: %s\n",
+					e, shellhook, strerror(e));
+			return RET_ERRNO(e);
+		}
+	} while( c != child );
+	if( WIFEXITED(status) ) {
+		if( WEXITSTATUS(status) == 0 ) {
+			if( verbose > 5 )
+				fprintf(stderr, "shell hook successfully returned!\n");
+			return RET_OK;
+		} else {
+			fprintf(stderr,
+"shell hook '%s' failed with exitcode %d!\n",
+					shellhook, (int)WEXITSTATUS(status));
+			return RET_ERROR;
+		}
+	} else {
+		fprintf(stderr,
+"shell hook '%s' terminated abnormally. (status is %x)!\n",
+				shellhook, status);
+		return RET_ERROR;
+	}
+	return RET_OK;
+}
+
 static retvalue calllisthooks(struct update_distribution *d) {
 	retvalue result,r;
 	struct update_target *target;
@@ -1396,13 +1524,24 @@ static retvalue calllisthooks(struct update_distribution *d) {
 		 * all (in case there are delete rules) */
 		for( uindex = target->indices ; uindex != NULL ;
 				uindex = uindex->next ) {
+			const struct update_pattern *p;
+
 			if( uindex->remote == NULL )
 				continue;
-			if( uindex->origin->pattern->listhook == NULL )
-				continue;
 			if( uindex->failed )
 				continue;
-			r = calllisthook(target, uindex);
+			p = uindex->origin->pattern;
+			while( p != NULL && p->listhook == NULL
+			                 && p->shellhook == NULL )
+				p = p->pattern_from;
+			if( p == NULL )
+				continue;
+			if( p->listhook != NULL )
+				r = calllisthook(target, uindex, p->listhook);
+			else {
+				assert( p->shellhook != NULL );
+				r = callshellhook(target, uindex, p->shellhook);
+			}
 			if( RET_WAS_ERROR(r) ) {
 				uindex->failed = true;
 				return r;
@@ -1430,7 +1569,7 @@ static retvalue updates_calllisthooks(struct update_distribution *distributions)
  *         (all the logic in upgradelist.c, this is only clue code)         *
  ****************************************************************************/
 
-static upgrade_decision ud_decide_by_pattern(void *privdata, const char *package, /*@null@*/const char *old_version, UNUSED(const char *new_version), const char *newcontrolchunk) {
+static upgrade_decision ud_decide_by_pattern(void *privdata, const struct target *target, const char *package, /*@null@*/const char *old_version, UNUSED(const char *new_version), const char *newcontrolchunk) {
 	const struct update_pattern *pattern = privdata, *p;
 	retvalue r;
 	upgrade_decision decision = UD_UPGRADE;
@@ -1470,7 +1609,8 @@ static upgrade_decision ud_decide_by_pattern(void *privdata, const char *package
 	while( p != NULL && !p->includecondition_set )
 		p = p->pattern_from;
 	if( p != NULL ) {
-		r = term_decidechunk(p->includecondition, newcontrolchunk);
+		r = term_decidechunktarget(p->includecondition,
+				newcontrolchunk, target);
 		if( RET_WAS_ERROR(r) )
 			return UD_ERROR;
 		if( r == RET_NOTHING ) {
@@ -1618,18 +1758,17 @@ static void updates_from_callback(void *privdata, const char **rule_p, const cha
 static retvalue updates_install(struct database *database, struct update_distribution *distribution) {
 	retvalue result,r;
 	struct update_target *u;
+	struct distribution *d = distribution->distribution;
 
-	assert( logger_isprepared(distribution->distribution->logger) );
+	assert( logger_isprepared(d->logger) );
 
 	result = RET_NOTHING;
 	for( u=distribution->targets ; u != NULL ; u=u->next ) {
 		if( u->nothingnew )
 			continue;
-		r = upgradelist_install(u->upgradelist,
-				distribution->distribution->logger,
-				database,
+		r = upgradelist_install(u->upgradelist, d->logger, database,
 				u->ignoredelete, updates_from_callback);
-		RET_UPDATE(distribution->distribution->status, r);
+		RET_UPDATE(d->status, r);
 		if( RET_WAS_ERROR(r) )
 			u->incomplete = true;
 		RET_UPDATE(result,r);
@@ -1638,6 +1777,10 @@ static retvalue updates_install(struct database *database, struct update_distrib
 		if( RET_WAS_ERROR(r) )
 			break;
 	}
+	if( RET_IS_OK(result) && d->tracking != dt_NONE ) {
+		r = tracking_retrack(database, d, false);
+		RET_ENDUPDATE(result, r);
+	}
 	return result;
 }
 
@@ -1825,7 +1968,7 @@ retvalue updates_update(struct database *database, struct update_distribution *d
 	/* Then get all packages */
 	if( verbose >= 0 )
 		printf("Calculating packages to get...\n");
-	r = downloadcache_initialize(database, mode, reserveddb, reservedother, &cache);
+	r = downloadcache_initialize(mode, reserveddb, reservedother, &cache);
 	if( !RET_IS_OK(r) ) {
 		aptmethod_shutdown(run);
 		RET_UPDATE(result,r);
@@ -1862,7 +2005,7 @@ retvalue updates_update(struct database *database, struct update_distribution *d
 	}
 	if( verbose >= 0 )
 		printf("Getting packages...\n");
-	r = aptmethod_download(run, database);
+	r = aptmethod_download(run);
 	RET_UPDATE(result,r);
 	r = downloadcache_free(cache);
 	RET_ENDUPDATE(result,r);
@@ -2096,6 +2239,7 @@ retvalue updates_predelete(struct database *database, struct update_distribution
 	if( verbose >= 0 )
 		printf("Removing obsolete or to be replaced packages...\n");
 	for( d=distributions ; d != NULL ; d=d->next) {
+		struct distribution *dd = d->distribution;
 		struct update_target *u;
 
 		for( u=d->targets ; u != NULL ; u=u->next ) {
@@ -2111,9 +2255,8 @@ retvalue updates_predelete(struct database *database, struct update_distribution
 				continue;
 			}
 			r = upgradelist_predelete(u->upgradelist,
-					d->distribution->logger,
-					database);
-			RET_UPDATE(d->distribution->status, r);
+					dd->logger, database);
+			RET_UPDATE(dd->status, r);
 			if( RET_WAS_ERROR(r) )
 				u->incomplete = true;
 			RET_UPDATE(result,r);
@@ -2121,6 +2264,10 @@ retvalue updates_predelete(struct database *database, struct update_distribution
 			u->upgradelist = NULL;
 			if( RET_WAS_ERROR(r) )
 				return r;
+			if( RET_IS_OK(result) && dd->tracking != dt_NONE ) {
+				r = tracking_retrack(database, dd, false);
+				RET_ENDUPDATE(result, r);
+			}
 		}
 	}
 	logger_wait();
